"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts = __importStar(require("typescript"));
var replace_1 = require("./replace");
var configuration_1 = __importDefault(require("./configuration"));
var typescript_1 = require("typescript");
var createPx2rem_1 = __importDefault(require("./createPx2rem"));
var _px2rem;
var _used = false;
function isStyledTagged(node) {
    var tag = node.tag;
    if (ts.isIdentifier(tag)) {
        return isStyled(tag);
    }
    else if (ts.isCallExpression(tag)) {
        return isStyledFunction(tag);
    }
    else if (ts.isPropertyAccessExpression(tag)) {
        return isStyledMember(tag);
    }
    return false;
}
function isStyledFunction(call) {
    var expression = call.expression;
    if (typescript_1.isPropertyAccessExpression(expression)) {
        return isStyledMember(expression);
    }
    else if (ts.isCallExpression(expression)) {
        return isStyledFunction(expression);
    }
    else if (ts.isIdentifier(expression)) {
        return isStyled(expression);
    }
    return false;
}
function isStyledMember(node) {
    if (ts.isIdentifier(node.expression)) {
        return isStyled(node.expression);
    }
    else if (ts.isPropertyAccessExpression(node.expression)) {
        return isStyledMember(node.expression);
    }
    return false;
}
function isStyled(id) {
    var tags = configuration_1.default.config.tags;
    return Array.isArray(tags) && tags.includes(id.text);
}
function createIfDifference(text, create, origin) {
    var replaced = replace_1.replace(text);
    if (replaced !== text) {
        var newNode = create(replaced);
        newNode.parent = origin.parent;
        return newNode;
    }
    return origin;
}
function isPureExpression(expression) {
    return (ts.isIdentifier(expression) ||
        ts.isLiteralExpression(expression) ||
        ts.isPropertyAccessExpression(expression) ||
        ts.isElementAccessExpression(expression) ||
        ts.isCallExpression(expression) ||
        ts.isBinaryExpression(expression));
}
function createCallPx2rem(px2rem) {
    var expression = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        expression[_i - 1] = arguments[_i];
    }
    _used = true;
    return ts.createCall(px2rem, undefined, expression);
}
function createTemplateSpanExpressionVisitor(context, px2rem) {
    var visitor = function (node) {
        if (typescript_1.isArrowFunction(node)) {
            if (ts.isBlock(node.body)) {
                return ts.updateArrowFunction(node, node.modifiers, node.typeParameters, node.parameters, node.type, node.equalsGreaterThanToken, ts.createCall(px2rem, undefined, [
                    ts.createArrowFunction(undefined, undefined, [], undefined, typescript_1.createToken(ts.SyntaxKind.EqualsGreaterThanToken), node.body),
                ]));
            }
            else {
                return ts.updateArrowFunction(node, node.modifiers, node.typeParameters, node.parameters, node.type, node.equalsGreaterThanToken, createCallPx2rem(px2rem, node.body));
            }
        }
        else if (ts.isConditionalExpression(node)) {
            if (isPureExpression(node.whenTrue) && isPureExpression(node.whenFalse)) {
                return ts.updateConditional(node, node.condition, node.questionToken, createCallPx2rem(px2rem, node.whenTrue), node.colonToken, createCallPx2rem(px2rem, node.whenFalse));
            }
            else if (isPureExpression(node.whenTrue)) {
                node = ts.updateConditional(node, node.condition, node.questionToken, createCallPx2rem(px2rem, node.whenTrue), node.colonToken, node.whenFalse);
            }
            else if (isPureExpression(node.whenFalse)) {
                node = ts.updateConditional(node, node.condition, node.questionToken, node.whenTrue, node.colonToken, createCallPx2rem(px2rem, node.whenFalse));
            }
        }
        else if (ts.isFunctionExpression(node)) {
            return ts.createArrowFunction(undefined, undefined, [
                ts.createParameter(undefined, undefined, ts.createToken(ts.SyntaxKind.DotDotDotToken), 'args', undefined, undefined),
            ], undefined, ts.createToken(ts.SyntaxKind.EqualsGreaterThanToken), createCallPx2rem(px2rem, node, ts.createSpread(ts.createIdentifier('args'))));
        }
        return ts.visitEachChild(node, visitor, context);
    };
    return visitor;
}
function createTemplateSpanVisitor(context) {
    var visitor = function (node) {
        if (_px2rem) {
            if (ts.isTemplateSpan(node) && node.literal.text && node.literal.text.startsWith('px')) {
                var text = node.literal.text.replace(/^px/, '');
                if (isPureExpression(node.expression)) {
                    node = ts.updateTemplateSpan(node, createCallPx2rem(_px2rem, node.expression), ts.isTemplateMiddle(node.literal) ? ts.createTemplateMiddle(text) : ts.createTemplateTail(text));
                }
                else {
                    node = ts.updateTemplateSpan(node, ts.visitNode(node.expression, createTemplateSpanExpressionVisitor(context, _px2rem)), ts.isTemplateMiddle(node.literal) ? ts.createTemplateMiddle(text) : ts.createTemplateTail(text));
                }
            }
            return ts.visitEachChild(node, visitor, context);
        }
        else {
            return node;
        }
    };
    return visitor;
}
function createStyledVisitor(context) {
    var visitor = function (node) {
        if (ts.isNoSubstitutionTemplateLiteral(node)) {
            return createIfDifference(node.text, function (replaced) { return ts.createNoSubstitutionTemplateLiteral(replaced); }, node);
        }
        else if (ts.isTemplateHead(node)) {
            return createIfDifference(node.text, function (replaced) { return ts.createTemplateHead(replaced); }, node);
        }
        else if (ts.isStringLiteral(node)) {
            return createIfDifference(node.text, function (replaced) { return ts.createStringLiteral(replaced); }, node);
        }
        else if (ts.isTemplateTail(node)) {
            return createIfDifference(node.text, function (replaced) { return ts.createTemplateTail(replaced); }, node);
        }
        else if (ts.isTemplateMiddle(node)) {
            return createIfDifference(node.text, function (replaced) { return ts.createTemplateMiddle(replaced); }, node);
        }
        var result = ts.visitEachChild(node, visitor, context);
        if (configuration_1.default.config.transformRuntime) {
            return ts.visitNode(result, createTemplateSpanVisitor(context));
        }
        return result;
    };
    return visitor;
}
exports.transformerFactory = function (context) {
    var styledVisitor = createStyledVisitor(context);
    var visitor = function (node) {
        if (ts.isTaggedTemplateExpression(node) && isStyledTagged(node)) {
            return ts.visitNode(node, styledVisitor);
        }
        else if (ts.isCallExpression(node) && isStyledFunction(node)) {
            return ts.visitNode(node, styledVisitor);
        }
        return ts.visitEachChild(node, visitor, context);
    };
    return function (node) {
        if (configuration_1.default.config.transformRuntime) {
            _px2rem = ts.createUniqueName('px2rem');
        }
        else {
            _px2rem = undefined;
        }
        _used = false;
        var sourceFile = ts.visitNode(node, visitor);
        if (_used && _px2rem) {
            return ts.updateSourceFileNode(sourceFile, __spreadArrays(sourceFile.statements, [
                createPx2rem_1.default(_px2rem, configuration_1.default.config),
            ]));
        }
        return sourceFile;
    };
};
//# sourceMappingURL=transform.js.map